% MAIN PROGRAM FOR "TAXATION OF FINANCIAL SECTOR"
% PHILIPPON, 2007

% Programs called: calib, equi0, equi, h, g, c1rat, U.
clear all ; close all ; t0 = clock ;

global period hcal Rcal bcal invshare edushare rho beta alpha gamma theta q z s he hX taufi taux tauw px e0 x0

options = optimset('Display','off') ;

% =========================================================================
period = 25  ; %period in years
taufi = 0 ; taux = 0 ; tauw = 0 ; % note that "taux" is a subsidy
rho = 2 ; beta = (1/1.02)^period ; alpha = 0.6 ; 
hcal = 1.0182^period ; 
Rcal = 1.04^period ; 
bcal = 0.06 ; % Beware of confusion: h is growth rate. called 1+h in the paper.
invshare = 0.125 ; 
edushare = 0 ;
px=1 ; % relative price of investment

% z can be computed directly (see notes)
z = 0.8*(1-alpha)*hcal/invshare ;

% calibrate gamma, theta, q
gamma = 0.59 ; theta = 0.79 ; q = 0.95 ; s = 0 ; % initial guesses
ZZ0 = [gamma theta q] ;
ZZ = fsolve(@calib,ZZ0,options) ;
gamma = ZZ(1) ; theta=ZZ(2) ; q=ZZ(3);
disp('Check: these values should be close to 0')
disp(calib(ZZ)') 

disp('Key parameters of the model')
disp('    gamma     theta      q        z')
disp([gamma theta q z])

% ESTIMATE MODEL GIVEN CALIBRATED PARAMETERS
e0 = 0.1 ; x0 = 0.2 ; phi0 = 0.5 ; c10 = 0.1 ;
XX0 = [e0 x0 Rcal phi0 c10] ;
XX = fsolve(@equi0,XX0,options) ;
e0 = XX(1) ; x0 = XX(2) ; R0 = XX(3) ; phi0 = XX(4) ; c10 = XX(5) ;
growth = hcal ;
c1e0 = c10*c1rat(R0,phi0) ; % Euler+Indiff
c20 = c10*(beta*R0)^(1/rho)/growth ; % Euler
c2e0 = c1e0*(beta*R0/(1-phi0))^(1/rho)/growth ; % Euler
y0 = ((1-e0)*c20 +  e0*c2e0)/(1-alpha) ;
b0 = (z*x0.*px-growth*c2e0).*e0/q ;
n0 = 1 - b0 - e0 ;
k0 = e0*g(x0)/growth;
ytest0 = n0^alpha*k0^(1-alpha);

disp('Implied values for key macroeconomic ratios')
disp('    e*x.*px/y      R       b      K/Y')
disp([e0*x0.*px/y0    R0^(1/period)   b0    k0/y0*period])


% ESIMATE RELATIVE VALUES OF HE AND HX %
px=1; % forcing the relative price of investment by technology. note this affects the resource constraints
taux=-0.1; % using taxes to change the relative price of investment. no effect on resource constraints
XX0 = [e0 x0 Rcal phi0 c10] ;
XX = fsolve(@equi0,XX0,options) ;
eg = XX(1) ; xg = XX(2) ; Rg = XX(3) ; phig = XX(4) ; c1g = XX(5) ;
px=1;
taux=0;

taufi = 0.1 ; taux = 0 ; tauw = 0.1 ;
XX0 = [e0 x0 Rcal phi0 c10] ;
XX = fsolve(@equi0,XX0,options) ;
es = XX(1) ; xs = XX(2) ; Rs = XX(3) ; phis = XX(4) ; c1s = XX(5) ;
efrac = (es-e0)/e0 ;
taufi = 0 ; taux = 0 ; tauw = 0 ;

he0 = ( (hcal^(1/period)+0.1864/100)^period - hcal )/e0/0.1 ; %the partial derivative wrt e
hX0 = ( (hcal^(1/period)-0.0777/100)^period - hcal - he0*(eg-e0) )/(eg*xg-e0*x0) ; %total derivative wrt to experiment above

%----- ALL IMPACTS ARE FOR 10% CHANGE IN PRICE OF INVESTMENT OR IN e --%
he = he0; hX = hX0;
dgpred_relp = 100*( (hcal+ (h(eg,xg)-h(e0,x0)) )^(1/period) - hcal^(1/period) ) ; % predicted impact of investment price on growth
dgpred_e = 100*( ( hcal+ ( h(1.1*e0,x0/1.1)-h(e0,x0) ) )^(1/period) - hcal^(1/period) ) ; % predicted impact of e only
dgpred_s = 100*( ( hcal+ 0.1/efrac*( h(es,xs)-h(e0,x0) ) )^(1/period) - hcal^(1/period) ) ; % predicted by e caused by s

disp('Predicted impact of inv price and education on growth')
disp('   +10% px  +10% educ')
disp([ dgpred_relp  dgpred_s  ])



%=== TAXATION AND GROWTH ===%
% CONSIDER 6 CASES: TAX ON LABOR OR FINANCE, EESHARE = 0 OR >0

taufi = 0 ; taux = 0 ; tauw = 0 ;
NN = 20 ;
tauwvec = linspace(0,0.25,NN) ;
taufivec = linspace(-0.25,0.25,NN) ;
Uw = zeros(3,NN) ; ew = Uw ; xw = Uw ; Rw = Uw ; phiw=Uw; c1w=Uw; c2w=Uw;
Ufi = zeros(3,NN) ; efi = Ufi ; xfi = Ufi ; Rfi = Ufi ; phif=Ufi; c1fi=Ufi; c2fi=Ufi;
for i=1:NN
% TAX NON FINANCIAL SECTOR
    taufi = 0 ; tauw = tauwvec(i) ;
    % Romer economy
    he = 0; hX = hX0;
    XX = fsolve(@equi,XX0,options) ; 
    ew(1,i) = XX(1); xw(1,i)=XX(2); Rw(1,i)=XX(3); phiw(1,i) = XX(4); c1w(1,i) = XX(5) ;
    % Lucas economy
    he = he0; hX = 0;
    XX = fsolve(@equi,XX0,options) ;
    ew(2,i) = XX(1); xw(2,i)=XX(2); Rw(2,i)=XX(3); phiw(2,i) = XX(4); c1w(2,i) = XX(5) ;
    % Benchmark economy
    he = he0; hX = hX0;
    XX = fsolve(@equi,XX0,options) ;
    ew(3,i) = XX(1); xw(3,i)=XX(2); Rw(3,i)=XX(3); phiw(3,i) = XX(4); c1w(3,i) = XX(5) ;
    
% TAX FINANCIAL SECTOR
    tauw=0 ;  taufi = taufivec(i) ;
    % Romer
    he = 0; hX = hX0;
    XX = fsolve(@equi,XX0,options) ;
    efi(1,i) = XX(1); xfi(1,i)=XX(2); Rfi(1,i)=XX(3); phifi(1,i) = XX(4); c1fi(1,i) = XX(5) ;
    % Lucas
    he = he0; hX = 0;
    XX = fsolve(@equi,XX0,options) ;    
    efi(2,i) = XX(1); xfi(2,i)=XX(2); Rfi(2,i)=XX(3); phifi(2,i) = XX(4); c1fi(2,i) = XX(5) ;
    % Benchmark
    he = he0; hX = hX0;
    XX = fsolve(@equi,XX0,options) ;    
    efi(3,i) = XX(1); xfi(3,i)=XX(2); Rfi(3,i)=XX(3); phifi(3,i) = XX(4); c1fi(3,i) = XX(5) ;
end


%====== GROWTH RATES =========%

% Romer
he = 0; hX = hX0;
g_w_l = h(ew(1,:),xw(1,:)).^(1/period) ;
g_fi_l = h(efi(1,:),xfi(1,:)).^(1/period) ;

% Lucas
he = he0; hX = 0;
g_w_h = h(ew(2,:),xw(2,:)).^(1/period) ;
g_fi_h = h(efi(2,:),xfi(2,:)).^(1/period) ;

%Benchmark
he = he0; hX = hX0;
g_w_m = h(ew(3,:),xw(3,:)).^(1/period) ;
g_fi_m = h(efi(3,:),xfi(3,:)).^(1/period) ;

figure(1)
    plot(tauwvec,g_w_l,'-+',tauwvec,g_w_h,'-.',tauwvec,g_w_m) 
    xlabel('INDUSTRIAL LABOR INCOME TAX')
    ylabel('ANNUAL GROWTH RATE')
    legend('Romer (86) Economy','Lucas (88) Economy','Estimated Economy','Location','Best')
    
figure(2)
    plot(taufivec,g_fi_l,'-+',taufivec,g_fi_h,'-.', taufivec,g_fi_m) 
    xlabel('TAX OR SUBSIDY TO FINANCIAL SECTOR')
    ylabel('ANNUAL GROWTH RATE')
    legend('Romer (86) Economy','Lucas (88) Economy','Estimated Economy','Location','Best')
    
    
%======= WELFARE ========%
c2w = c1w.*(beta.*Rw).^(1/rho) ;
Uw = U(c1w,c2w) ;
c2fi = c1fi.*(beta.*Rfi).^(1/rho) ;
Ufi = U(c1fi,c2fi) ;


